/*

*	Address correspondence to Herman Aguinis, Dept. of Psychology,

*	University of Colorado at Denver, Denver, CO, 80217-3364; Internet:

*	haguinis@castle.cudenver.edu.

*/



#include <stdio.h>

#include <stdlib.h>

#include <string.h>

#include <ctype.h>

#include <dos.h>

#include <graphics.h>

#include <time.h>



#define FALSE 0

#define TRUE (!FALSE)

#define TYPECHAR '|'

#define DISPLAYTYPE 'D'

#define AGREETYPE 'A'

#define SATISFIEDTYPE 'S'

#define LINESIZE 80

#define NUMSIZE 6

#define MAXTEXT 400

#define GAP 2

#define LINEARWIDTH 0

#define LINEARLO 0.01

#define LINEARHI 0.99

#define PROMPT "?>"

#define POPTIME 50



FILE *questions= NULL, *datafile= NULL;

char InputLine[LINESIZE], *pos;

int LinePos;

int MaxX, MaxY, TWidth, THeight;

int TextTop, TextBottom;

int HitLeft, HitTop, HitRight, HitBottom, HitTX, HitTY;

int LinearLeft, LinearRight, LinearTop, LinearBottom;

int LinearTLeft, LinearTRight, LinearTY;

int mouse= FALSE;

int POPOUT;

const char *next= "NEXT";



/*

*	Hide the mouse without the distracting counters provided by the

*	mouse driver.

*/

void HideMouse(void)

{

	union REGS regs;



	if (mouse==TRUE) {

		regs.x.ax= 0x02;        /* hide the mouse pointer */

		int86(0x33, &regs, &regs);

	}

	mouse= FALSE;

	return;

}



/*

*	Show the mouse without the distracting counters provided by the

*	mouse driver.

*/

void ShowMouse(void)

{

	union REGS regs;



	if (mouse==FALSE) {

		regs.x.ax= 0x01;        /* show the mouse pointer */

		int86(0x33, &regs, &regs);

	}

	mouse= TRUE;

	return;

}



/*

*	Draw the next box in the center bottom of the screen. If h is TRUE

*	the box will appear highlighted, when H is FALSE a simple box will

*	be drawn.

*/

void DrawNextBox(int h)

{

	/* compute box dimentions */

	TWidth= textwidth(next);

	HitTX= MaxX/2 - TWidth/2;

	HitTY= MaxY-THeight-GAP*2;

	HitLeft= HitTX-GAP;

	HitTop= HitTY-GAP;

	HitRight= HitTX+TWidth+GAP;

	HitBottom= HitTY+THeight+GAP;

	/* draw the box in the right style */

	setfillstyle(SOLID_FILL, BLACK);

	bar(HitLeft, HitTop, HitRight, HitBottom);

	if (h == TRUE) {

		setfillstyle(SOLID_FILL, WHITE);

		bar(HitLeft, HitTop, HitRight, HitBottom);

		setcolor(BLACK);

		outtextxy(HitTX, HitTY, next);

	}

	else {

		setcolor(WHITE);

		rectangle(HitLeft, HitTop, HitRight, HitBottom);

		outtextxy(HitTX, HitTY, next);

	}

	return;

}



/*

*	Render the linear scale for  the user to select a point on. The two

*	string arguments are the labels for that should appear below the

*	line on the left and right sides.

*/

void DrawLinearScale(char *tleft, char *tright)

{

	/* compute dimentions for the line and text fields */

	TWidth= textwidth(tright);

	LinearTLeft= LinearLeft+GAP;

	LinearTRight= LinearRight-TWidth-GAP;

	LinearTY= LinearBottom+GAP;

	/* draw the line and text */

	setcolor(WHITE);

	outtextxy(LinearTLeft, LinearTY, tleft);

	outtextxy(LinearTRight, LinearTY, tright);

	setfillstyle(SOLID_FILL, WHITE);

	bar(LinearLeft, LinearTop, LinearRight, LinearBottom);

	return;

}



/*

*	Read a decimal number from the input buffer. pos should be pre-

*	inialized to the beginning of the number.

*/

int ReadNumber(void)

{

	int value= 0, i= 1;



	while (isdigit(pos[i]))

		value = 10*value + pos[i++]-'0';

	return value;

}



/*

*	Show the question text.

*/

char ShowQText(void)

{

	int tx;



	TextTop= THeight*1;	/* leave some white sapce at top of screen */

	tx= 0;

	HideMouse();

	cleardevice();	/* clear the graphics screen */

	setcolor(WHITE);

	for(TextBottom= TextTop; TextBottom < MAXTEXT; TextBottom += THeight*2) {

		/* get a line from the file */

		fgets(InputLine, LINESIZE, questions);

		/* remove newlines */

		if ((pos= strchr(InputLine, '\n')) != NULL)

			*pos= '\0';

		/* look for a input type field */

		if ((pos= strchr(InputLine, TYPECHAR)) == NULL) {

			/* no type field, output text */

			tx= MaxX/2 - textwidth(InputLine)/2;	/* center on line */

			outtextxy(tx, TextBottom, InputLine);

			continue;       /* keep reading text */

		}

		else {

			*pos= '\0';     /* remove input type symbol */

			tx= MaxX/2 - textwidth(InputLine)/2;	/* output text */

			outtextxy(tx, TextBottom, InputLine);

			break;  /* stop reading text */

		}

	}

	/* init linear values for later, linear position depends

	   on text length*/

	LinearLeft= MaxX*LINEARLO;

	LinearRight= MaxX*LINEARHI;

	LinearTop= TextBottom+(MaxY-TextBottom)/2;

	LinearBottom= LinearTop+LINEARWIDTH;

	return toupper(pos[1]); /* return input type */

}



/*

*	Get the users response from the next box. If mode is TRUE this

*	function will return TRUE when the user has clicked in the next

*	box or will return FALSE when the user has tried to move out of

*	the next box. When mode is FALSE the function will return only

*	after the user has clicked the next box.

*/

int GetNext(int mode) {

	union REGS regs;



	HideMouse();

	DrawNextBox(FALSE);

	do {    /* if a mouse button is down, wait for it to go up */

		regs.x.ax= 0x03;

		int86(0x33, &regs, &regs);

	} while (regs.x.bx != 0);

	ShowMouse();

	/* limit mouse movement to the next hit box */

	regs.x.ax= 0x07;        /* set x limits */

	regs.x.cx= HitLeft;

	regs.x.dx= HitRight;

	int86(0x33, &regs, &regs);

	regs.x.ax= 0x08;        /* set y limits */

	regs.x.cx= HitTop;

	regs.x.dx= HitBottom;

	int86(0x33, &regs, &regs);

	do {

		delay(POPTIME);		/* wait the same time on all machines */

		regs.x.ax= 0x0b;	/* read mouse counters */

		int86(0x33, &regs, &regs);

		if ((signed)regs.x.dx < -POPOUT && mode==TRUE)

			return FALSE;	/* exit if we moved fast enough to pop out */

		regs.x.ax= 0x05;	/* get mouse button status */

		int86(0x033, &regs, &regs);

	} while (regs.x.ax != 1);	/* wait for left button press */

	DrawNextBox(TRUE); 		/* hightlight the next box */

	return TRUE;	/* exit, goto next screen */

}



/*

*	Get a number from the user. The value entered wil be accepted only

*	if it is between 1 and the max passed to the function. Simple editing

*	is provided by the backspace key.

*/

long GetNumber(int max) {

	int i, c;

	long value;

	char buffer[NUMSIZE];

	char *cont= "Enter a number and press ENTER.";

	/* clear the entire input buffer */

	restart:

	HideMouse();

	/* put instructions at bottom of the screen */

	setcolor(WHITE);

	outtextxy(MaxX/2-textwidth(cont)/2, MaxY-THeight-GAP*5, cont);

	for(i= NUMSIZE;;) {	/* initalize i to clear buffer */

		/* clear input buffer if too many digits typed without an enter */

		if (i>=NUMSIZE) {

			for(i= 0; i<NUMSIZE; i++)

				buffer[i]= '\0';

			i= 0;

		}

		/* erase old value entered */

		setfillstyle(SOLID_FILL, BLACK);

		bar(0, LinearTop, getmaxx(), LinearTop+THeight);

		/* Draw prompt and the value entered so far */

		outtextxy(MaxX/2-textwidth(PROMPT), LinearTop, PROMPT);

		outtextxy(MaxX/2, LinearTop, buffer);

		while(!kbhit()) {       /* wait for keypress */

			int timer;	/* timer is used to make cursor flash */

			timer= (int) (clock()/(CLK_TCK/3));

			setfillstyle(SOLID_FILL, timer%2 ? BLACK : WHITE);

			bar(MaxX/2+textwidth("#")*i, LinearTop,

				MaxX/2+textwidth("#")*(i+1), LinearTop+THeight);

		}

		c= getch();

		if(isdigit(c))  /* if char is a digit put it in the string */

			buffer[i++]= c;

		else if (c==0x08 && i>0)        /* backspace erases last char */

			buffer[--i]='\0';

		if (c == '\r')

			break;  /* break out of for loop */

	}

	value= atol(buffer);

	if (value<1 || value>max)

		goto restart;

	return value;

}



/*

*   Get the users response to a linear value selection.

*/

long GetLinear(char *leftstr, char *rightstr) {

	union REGS regs;

	float value= 9.9999999;



	/* Draw the screen */

	HideMouse();

	DrawLinearScale(leftstr, rightstr);

	DrawNextBox(FALSE);

	ShowMouse();

	do {    /* if a mouse button is down, wait for it to go up */

		regs.x.ax= 0x03;

		int86(0x33, &regs, &regs);

	} while (regs.x.bx != 0);

	for(;;) {

		/* limit mouse movement to the linear scale */

		regs.x.ax= 0x07;        /* set x limits */

		regs.x.cx= LinearLeft;

		regs.x.dx= LinearRight;

		int86(0x33, &regs, &regs);

		regs.x.ax= 0x08;        /* set y limits */

		regs.x.cx= LinearTop;

		regs.x.dx= LinearBottom;

		int86(0x33, &regs, &regs);

		regs.x.ax= 0x03;    /* get mouse button info */

		int86(0x33, &regs, &regs);

		if (regs.x.bx == 1) {	/* only when left button is pressed */

			/* compute value between 0 and 1 */

			value= (float)(regs.x.cx-LinearLeft)/(float)(LinearRight-LinearLeft);

			value= value<0 ? 0 : value;

			value= value>1 ? 1 : value;

			/* erase old tick mark */

			setfillstyle(SOLID_FILL, BLACK);

			bar(LinearLeft, LinearTop-7, LinearRight, LinearTop-1);

			/* draw new tick mark */

			DrawLinearScale(leftstr, rightstr);

			moveto(regs.x.cx, LinearTop-7);

			lineto(regs.x.cx, LinearTop);

		}

		delay(POPTIME);     /* wait the same time on all machiens */

		regs.x.ax= 0x0b;	/* read mouse counters */

		int86(0x33, &regs, &regs);

		if (value<=1 && (signed)regs.x.dx>POPOUT && GetNext(TRUE))

			break;

	}

	return value*100000;

}



/*

*	Get a response from the user and record in to the datafile.

*/

void RecordResponce(char c)

{

	/* wrap line if needed */

	if (LinePos>60)

		LinePos= fprintf(datafile, "\n       ") - 1;



	if (c == DISPLAYTYPE)

		GetNext(FALSE);

	else if (c == AGREETYPE)

		LinePos += fprintf(datafile, "%06li ", GetLinear("AGREE", "DISAGREE"));

	else if (c == SATISFIEDTYPE)

		LinePos += fprintf(datafile, "%06li ", GetLinear("SATISFIED", "DISSATISFIED"));

	else if (isdigit(c))

		/* read a number from the question file, use as max

		   for user input */

		LinePos += fprintf(datafile, "%06li ", GetNumber(ReadNumber()));

	else {

		cleardevice();

		outtextxy(0, 0, "Unknown input type in question file.");

		outtextxy(0, 50, "Press any key to exit.");

		while(!kbhit());

		getch();

		exit(200);

	}

	return;

}



/*

*	Process questions in the question file until eof.

*/

void Process(void)

{

	int c;



	LinePos= 0;

	fputc('\n', datafile);

	while ((c= fgetc(questions)) != EOF) {

		if (kbhit() && getch() == 0x1b)

			break;

		ungetc(c, questions);

		RecordResponce(ShowQText());

	}

	return;

}



/*

*	Close shop when exiting program

*/

void CloseAll(void) {

	union REGS regs;

	/* close everything opened */

	int86(0x33, &regs, &regs);

	closegraph();

	fclose(questions);

	fclose(datafile);

	closegraph();

	return;

}



int main(int argc, char **argv)

{

	int gdriver= DETECT, gmode, errcode;

	union REGS regs;

	struct SREGS sregs;



	/* check args and open files */

	if (argc < 3) {

		printf("usage: %s questions.txt datafile.txt [s_index]\n", argv[0]);

		exit(10);

	}

	if ((questions= fopen(argv[1],"r")) == NULL) {

		printf("Error opening question file\n");

		exit(11);

	}

	if ((datafile= fopen(argv[2],"a+")) == NULL) {

		printf("Error opening question file\n");

		fclose(questions);

		exit(12);

	}

	if (argc==4)

		POPOUT= atoi(argv[3]);

	else

		POPOUT= 10;



	/* initialize graphics mode, alwase open on a 640x200 screen */

	gdriver= registerbgidriver(EGAVGA_driver);

	if (gdriver == EGA)

		gmode= EGALO;

	else if (gdriver == VGA)

		gmode= VGALO;

	else if ((gdriver= registerbgidriver(CGA_driver)) != CGA )

		gmode= CGAHI;

	else {

		printf("Error initalizing graphics systems, no bgi driver\n");

		fclose(questions);

		fclose(datafile);

		exit(13);

	}

	initgraph(&gdriver, &gmode, "");

	if ((errcode= graphresult()) != grOk) {

		printf("Graphics error: %s\n", grapherrormsg(errcode));

		fclose(questions);

		fclose(datafile);

		exit(14);

	}

	/* initalize some constants based on the video mode */

	MaxX= getmaxx();

	MaxY= getmaxy();

	settextstyle(DEFAULT_FONT, HORIZ_DIR, 1);

	THeight= textheight("TEXT");



	/* initalize the mouse */

	regs.x.ax= 0x3533;      /* check the value of interupt vector #0x33 */

	intdosx(&regs, &regs, &sregs);

	if ((regs.x.bx | sregs.es) == 0) {

		printf("No mouse driver installed\n");

		closegraph();

		fclose(questions);

		fclose(datafile);

		exit(15);

	}

	regs.x.ax= 0x00;        /* initalize the mouse driver */

	int86(0x33, &regs, &regs);



	/* Now process the questions */

	atexit(CloseAll);  /* if error occurs during process, still close shop */

	Process();

	return 0;

}


